@@ -1,17 +1,14 @@
 {-# LANGUAGE CPP #-}
-{-|
-Definitions of lazy Deque.
 
-The typical `toList` and `fromList` conversions are provided by means of
-the `Foldable` and `IsList` instances.
--}
-module Deque.Lazy.Defs
-where
+-- |
+-- Definitions of lazy Deque.
+--
+-- The typical `toList` and `fromList` conversions are provided by means of
+-- the `Foldable` and `IsList` instances.
+module Deque.Lazy.Defs where
 
-import Control.Monad (fail)
-import Deque.Prelude hiding (tail, init, last, head, null, dropWhile, takeWhile, reverse, filter, take)
 import qualified Data.List as List
-import qualified Deque.Prelude as Prelude
+import Deque.Prelude hiding (dropWhile, filter, head, init, last, null, reverse, tail, take, takeWhile)
 
 -- |
 -- Lazy double-ended queue (aka Dequeue or Deque) based on head-tail linked list.
@@ -33,76 +30,84 @@ filter predicate (Deque consList snocList) = Deque (List.filter predicate consLi
 -- \(\mathcal{O}(n)\).
 -- Leave only the specified amount of first elements.
 take :: Int -> Deque a -> Deque a
-take amount (Deque consList snocList) = let
-  newConsList = let
-    buildFromConsList amount = if amount > 0
-      then \ case
-        head : tail -> head : buildFromConsList (pred amount) tail
-        _ -> buildFromSnocList amount (List.reverse snocList)
-      else const []
-    buildFromSnocList amount = if amount > 0
-      then \ case
-        head : tail -> head : buildFromSnocList (pred amount) tail
-        _ -> []
-      else const []
-    in buildFromConsList amount consList
-  in Deque newConsList []
+take amount (Deque consList snocList) =
+  let newConsList =
+        let buildFromConsList amount =
+              if amount > 0
+                then \case
+                  head : tail -> head : buildFromConsList (pred amount) tail
+                  _ -> buildFromSnocList amount (List.reverse snocList)
+                else const []
+            buildFromSnocList amount =
+              if amount > 0
+                then \case
+                  head : tail -> head : buildFromSnocList (pred amount) tail
+                  _ -> []
+                else const []
+         in buildFromConsList amount consList
+   in Deque newConsList []
 
 -- |
 -- \(\mathcal{O}(n)\).
 -- Drop the specified amount of first elements.
 drop :: Int -> Deque a -> Deque a
-drop amount (Deque consList snocList) = let
-  buildFromConsList amount = if amount > 0
-    then \ case
-      _ : tail -> buildFromConsList (pred amount) tail
-      _ -> buildFromSnocList amount (List.reverse snocList)
-    else \ tail -> Deque tail snocList
-  buildFromSnocList amount = if amount > 0
-    then \ case
-      _ : tail -> buildFromSnocList (pred amount) tail
-      _ -> Deque [] []
-    else \ tail -> Deque tail []
-  in buildFromConsList amount consList
+drop amount (Deque consList snocList) =
+  let buildFromConsList amount =
+        if amount > 0
+          then \case
+            _ : tail -> buildFromConsList (pred amount) tail
+            _ -> buildFromSnocList amount (List.reverse snocList)
+          else \tail -> Deque tail snocList
+      buildFromSnocList amount =
+        if amount > 0
+          then \case
+            _ : tail -> buildFromSnocList (pred amount) tail
+            _ -> Deque [] []
+          else \tail -> Deque tail []
+   in buildFromConsList amount consList
 
 -- |
 -- \(\mathcal{O}(n)\).
 -- Leave only the first elements satisfying the predicate.
 takeWhile :: (a -> Bool) -> Deque a -> Deque a
-takeWhile predicate (Deque consList snocList) = let
-  newConsList = List.foldr
-    (\ a nextState -> if predicate a
-      then a : nextState
-      else [])
-    (List.takeWhile predicate (List.reverse snocList))
-    consList
-  in Deque newConsList []
+takeWhile predicate (Deque consList snocList) =
+  let newConsList =
+        List.foldr
+          ( \a nextState ->
+              if predicate a
+                then a : nextState
+                else []
+          )
+          (List.takeWhile predicate (List.reverse snocList))
+          consList
+   in Deque newConsList []
 
 -- |
 -- \(\mathcal{O}(n)\).
 -- Drop the first elements satisfying the predicate.
 dropWhile :: (a -> Bool) -> Deque a -> Deque a
-dropWhile predicate (Deque consList snocList) = let
-  newConsList = List.dropWhile predicate consList
-  in case newConsList of
-    [] -> Deque (List.dropWhile predicate (List.reverse snocList)) []
-    _ -> Deque newConsList snocList
+dropWhile predicate (Deque consList snocList) =
+  let newConsList = List.dropWhile predicate consList
+   in case newConsList of
+        [] -> Deque (List.dropWhile predicate (List.reverse snocList)) []
+        _ -> Deque newConsList snocList
 
 -- |
 -- \(\mathcal{O}(n)\).
 -- Perform `takeWhile` and `dropWhile` in a single operation.
 span :: (a -> Bool) -> Deque a -> (Deque a, Deque a)
 span predicate (Deque consList snocList) = case List.span predicate consList of
-  (consPrefix, consSuffix) -> if List.null consSuffix
-    then case List.span predicate (List.reverse snocList) of
-      (snocPrefix, snocSuffix) -> let
-        prefix = Deque (consPrefix <> snocPrefix) []
-        suffix = Deque snocSuffix []
-        in (prefix, suffix)
-    else let
-      prefix = Deque consPrefix []
-      suffix = Deque consSuffix snocList
-      in (prefix, suffix)
+  (consPrefix, consSuffix) ->
+    if List.null consSuffix
+      then case List.span predicate (List.reverse snocList) of
+        (snocPrefix, snocSuffix) ->
+          let prefix = Deque (consPrefix <> snocPrefix) []
+              suffix = Deque snocSuffix []
+           in (prefix, suffix)
+      else
+        let prefix = Deque consPrefix []
+            suffix = Deque consSuffix snocList
+         in (prefix, suffix)
 
 -- |
 -- \(\mathcal{O}(1)\), occasionally \(\mathcal{O}(n)\).
@@ -161,10 +166,10 @@ unsnoc (Deque consList snocList) = case snocList of
 -- |
 -- \(\mathcal{O}(n)\).
 prepend :: Deque a -> Deque a -> Deque a
-prepend (Deque consList1 snocList1) (Deque consList2 snocList2) = let
-  consList = consList1
-  snocList = snocList2 ++ foldl' (flip (:)) snocList1 consList2
-  in Deque consList snocList
+prepend (Deque consList1 snocList1) (Deque consList2 snocList2) =
+  let consList = consList1
+      snocList = snocList2 ++ foldl' (flip (:)) snocList1 consList2
+   in Deque consList snocList
 
 -- |
 -- \(\mathcal{O}(1)\).
@@ -173,7 +178,7 @@ reverse :: Deque a -> Deque a
 reverse (Deque consList snocList) = Deque snocList consList
 
 -- |
--- \(\mathcal{O}(1)\). 
+-- \(\mathcal{O}(1)\).
 -- Check whether deque is empty.
 null :: Deque a -> Bool
 null (Deque consList snocList) = List.null snocList && List.null consList
@@ -187,7 +192,7 @@ head = fmap fst . uncons
 -- |
 -- \(\mathcal{O}(1)\), occasionally \(\mathcal{O}(n)\).
 -- Keep all elements but the first one.
--- 
+--
 -- In case of empty deque returns an empty deque.
 tail :: Deque a -> Deque a
 tail = fromMaybe <$> id <*> fmap snd . uncons
@@ -195,7 +200,7 @@ tail = fromMaybe <$> id <*> fmap snd . uncons
 -- |
 -- \(\mathcal{O}(1)\), occasionally \(\mathcal{O}(n)\).
 -- Keep all elements but the last one.
--- 
+--
 -- In case of empty deque returns an empty deque.
 init :: Deque a -> Deque a
 init = fromMaybe <$> id <*> fmap snd . unsnoc
@@ -206,11 +211,10 @@ init = fromMaybe <$> id <*> fmap snd . unsnoc
 last :: Deque a -> Maybe a
 last = fmap fst . unsnoc
 
-
-instance Eq a => Eq (Deque a) where
+instance (Eq a) => Eq (Deque a) where
   (==) a b = toList a == toList b
 
-instance Show a => Show (Deque a) where
+instance (Show a) => Show (Deque a) where
   show = show . toList
 
 instance Semigroup (Deque a) where
@@ -234,22 +238,22 @@ deriving instance Functor Deque
 
 instance Applicative Deque where
   pure a = Deque [] [a]
-  (<*>) (Deque fnConsList fnSnocList) (Deque argConsList argSnocList) = let
-    consList = let
-      fnStep fn resultConsList = let
-        argStep arg = (:) (fn arg)
-        in foldr argStep (foldr argStep resultConsList (List.reverse argSnocList)) argConsList
-      in foldr fnStep (foldr fnStep [] (List.reverse fnSnocList)) fnConsList 
-    in Deque consList []
+  (<*>) (Deque fnConsList fnSnocList) (Deque argConsList argSnocList) =
+    let consList =
+          let fnStep fn resultConsList =
+                let argStep arg = (:) (fn arg)
+                 in foldr argStep (foldr argStep resultConsList (List.reverse argSnocList)) argConsList
+           in foldr fnStep (foldr fnStep [] (List.reverse fnSnocList)) fnConsList
+     in Deque consList []
 
 instance Monad Deque where
   return = pure
-  (>>=) (Deque aConsList aSnocList) k = let
-    consList = let
-      aStep a accBConsList = case k a of
-        Deque bConsList bSnocList -> bConsList <> foldl' (flip (:)) accBConsList bSnocList
-      in foldr aStep (foldr aStep [] (List.reverse aSnocList)) aConsList
-    in Deque consList []
+  (>>=) (Deque aConsList aSnocList) k =
+    let consList =
+          let aStep a accBConsList = case k a of
+                Deque bConsList bSnocList -> bConsList <> foldl' (flip (:)) accBConsList bSnocList
+           in foldr aStep (foldr aStep [] (List.reverse aSnocList)) aConsList
+     in Deque consList []
 #if !(MIN_VERSION_base(4,13,0))
   fail = const mempty
 #endif
@@ -273,9 +277,11 @@ instance IsList (Deque a) where
   toList (Deque consList snocList) = consList <> List.reverse snocList
 
 deriving instance Generic (Deque a)
+
 deriving instance Generic1 Deque
 
-instance Hashable a => Hashable (Deque a)
+instance (Hashable a) => Hashable (Deque a)
+
+instance (NFData a) => NFData (Deque a)
 
-instance NFData a => NFData (Deque a)
 instance NFData1 Deque
